# -*- coding: utf-8 -*-

import os

from httpkit.service.input import HTTPInputService
from httpkit.service.query import HTTPQueryService
from dreamweavertemplate.service import DreamweaverTemplateService
from authtkt import handle, get_default_handler_configuration, AuthTkt
from flows.framework.application import not_found
from flows.provider import start_flow, ServiceProvider
from bn import AttributeDict

def validate_sub(flow):
    # Replace this to check the username and password provided...
    flow.ext.username
    flow.ext.password
    raise Exception('Replace this with your own code')

def config(flow, name):
    # In this case no extra configuration is required
    flow.config[name] = {}
    return flow.config[name] 

def make_component(
    name,
    validate_sub=validate_sub,
    extra_available_services = None,
    extra_required_services = None,
):

    def run(flow):
        # Expose any event handlers
        flow['ext'] = AttributeDict()

        # Load the Apache config from the file specified by MOD_AUTH_TKT_CONF
        at = AuthTkt(conf=flow.config.option_group['auth']['filename'])
   
        # Use the default handler configuration but change the validate_sub. 
        conf = get_default_handler_configuration()
        conf['validate_sub'] = validate_sub

        # Obtain the mode from the PATH_INFO
        ext = flow.http.environ['PATH_INFO'][1:].split('/')[0]
        mode=flow.http.environ['PATH_INFO'][len(ext)+2:]
        base = '/%s/%s'%(ext, mode)
        if mode not in ['login', 'logout', 'autologin']:
            return not_found(flow)
        result = handle(
            at, 
            conf, 
            mode=mode, 
            base=base,
            flow=flow,
        )
        data = ''.join(flow.http.response.body)
        template_file = os.path.join(
            flow.config.app.app_dir, 
            'static', 
            'Templates',
            flow.config.option_group['auth'].get('template', 'error.dwt')
        )
        if os.path.exists(template_file):
            if '<h1>' in data:
                title = data[data.find('<h1>')+4:data.find('</h1>')]
                form = data[data.find('</h1>')+5:data.find('</form>')+7]
            else:
                title = data
                form = ''
            template = {}
            template['heading'] = title
            template['doctitle'] = '<title>%s</title>'%title
            template['content'] = form
            flow.http.response.body = flow.template.render(
                flow.config.option_group['auth'].get('template', 'error.dwt'),
                template,
            )
        else:
            flow.http.response.body = [ data or 'problem']

    # Customise this to specify all the services to start
    required_services = ['http.input', 'http.query', 'template'] + \
       (extra_required_services or [])

    # Customise this to specify all the available services
    available_services = extra_available_services or {}
    available_services.update({
        'http.input': HTTPInputService,
        'http.query': HTTPQueryService,
        'template': DreamweaverTemplateService,
    })

    # This is always the same:
    def on_load_component(flow):
        flow.config[name] = config(flow, name)
        def dispatch_contact_form(flow):
            f = AttributeDict(flow.copy())
            del f['provider']
            #if f.config.has_key('ext'):
            #    raise Exception(
            #        'Extension configuration is already present, perhaps the '
            #        '%r extension forgot to remove it?' % f.config.ext
            #    )
            #else:
            f.config['ext'] = flow.config[name]
            start_flow(
                run,
                provider = ServiceProvider(
                    existing_services = f,
                    available_services = available_services,
                ),
                required_services=required_services,
            )
        return dispatch_contact_form
    return on_load_component










"""\
This is a sample implementation of a login WSGI handler to work with Apache
mod_wsgi and mod_auth_tkt.

You should customise ``validate_sub()`` to determine which logins are allowed.

The PATH_INFO environment variable is used to decide whether we are in 
login, logout or autologin mode. If this script is set up to be called
at the ``/signin`` URL then ``/signin/login`` would trigger a login, 
``/signin/logout`` would trigger a logout etc.

If you are using this as part of an Apache mod_wsgi handler you will want
to include the following standard lines:

::

    # Choose the Python installation where AuthTkt and its dependencies are
    # installed and set up a place where eggs can be cached
    import os
    os.environ['PYTHON_EGG_CACHE'] = '/tmp'
    import site
    site.addsitedir('/path/to/env/lib/python2.5/site-packages')

"""

#import os
#
#from flows.framework.application import not_found
#from dreamweavertemplate import DWT
#from authtkt import handle, get_default_handler_configuration, AuthTkt
#import usermanager.driver.base
#
#
#def make_component(name, validate_sub):
#
#    def on_load_component(flow):
#        def auth_component(flow):
#            if not flow.has_key('http.input'):
#                flow.provider.start('http.input')
#            if not flow.has_key('http.query'):
#                flow.provider.start('http.query')
#        return auth_component
#
#
#
